/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm.fieldplan;

import nl.cloudfarming.client.farm.model.FieldPlanManager;
import nl.cloudfarming.client.farm.model.CropField;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.farm.model.*;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@Messages({"FieldPlan name prefix=Field plan-"})
public class FieldPlanManagerImpl implements FieldPlanManager {

    public static final String FIELDPLAN_OBJECT_NAME = "data";
    public static final String PRODUCTION_UNIT_OBJECT_NAME = "data";
    public static final String UNOCCUPIED_FOLDER_NAME = "unoccupied";
    public static final String FARM_FIELDPLANS_DIR = "fieldplans";
    private final DataFolder fieldPlansFolder;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.farm.fieldplan.FieldPlanManagerImpl");

    public FieldPlanManagerImpl(DataFolder fieldPlansFolder) throws IOException {
        this.fieldPlansFolder = fieldPlansFolder;
    }

    /**
     *
     * @param fieldPlan
     * @param fieldPlanFolder
     */
    @Override
    public void save(FieldPlan fieldPlan, FileObject fieldPlanFolder) throws IOException {
        LOGGER.log(Level.FINEST, "saving fieldplan data to folder {0}", fieldPlanFolder.getName());
        try (ObjectOutputStream oos = new ObjectOutputStream(fieldPlanFolder.createData(FIELDPLAN_OBJECT_NAME).getOutputStream())) {
            oos.writeObject(fieldPlan);
        }
    }

    /**
     * loads a fieldplan from a fileobject
     *
     * @param folder
     * @return
     */
    @Override
    public FieldPlan loadFieldPlan(FileObject fieldPlanFolder) throws IOException {
        LOGGER.log(Level.FINEST, "loading fieldplan data from folder {0}", fieldPlanFolder.getName());
        FileObject dataObject = fieldPlanFolder.getFileObject(FIELDPLAN_OBJECT_NAME);
        assert dataObject != null : "No dataobject found in provided folder!";

        FieldPlan fieldPlan = null;
        try (ObjectInputStream ois = new ObjectInputStream(dataObject.getInputStream())) {
            fieldPlan = (FieldPlan) ois.readObject();
        } catch (ClassNotFoundException | IOException ex) {
            Exceptions.printStackTrace(ex);
            throw new DataObjectNotFoundException(dataObject);
        }
        return fieldPlan;
    }

    /**
     * Add a new FieldPlan and save it
     *
     * @param harvestYear the harvestYear of the new fieldPlan
     * @param farmProject parent folder to store the new fieldplan in
     * @return the newly created fieldPlan
     */
    @Override
    public FileObject addFieldPlan(String name, Integer harvestYear) {
        LOGGER.log(Level.FINEST, "adding fieldplan with name {0} and year {1}", new Object[]{name, harvestYear});
        try {
            FieldPlan fieldPlan = new FieldPlan();
            fieldPlan.setHarvestYear(harvestYear);
            fieldPlan.setName(name);
            String fieldPlanFolderName = FileUtil.findFreeFolderName(fieldPlansFolder.getPrimaryFile(), fieldPlan.getName() + "-" + fieldPlan.getHarvestYear());
            FileObject fieldPlanFolder = fieldPlansFolder.getPrimaryFile().createFolder(fieldPlanFolderName);
            save(fieldPlan, fieldPlanFolder);
            return fieldPlanFolder;
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
            LOGGER.log(Level.SEVERE, "Failed to add fieldplan {0}", ex.getMessage());
        }
        return null;
    }

    /**
     * Create a new fieldplan and save it in the provided parent folder.
     * cropFields are stored in an unoccupied folder of the created fieldplan
     *
     * @param name
     * @param harvestYear
     * @param farmFolder
     * @param cropFields
     * @return
     */
    @Override
    public FileObject addFieldPlan(String name, Integer harvestYear, List<CropField> cropFields) {
        LOGGER.log(Level.FINEST, "adding fieldplan with name {0}, year {1} and {2} cropfields", new Object[]{name, harvestYear, cropFields.size()});
        FileObject fieldPlanFolder = addFieldPlan(name, harvestYear);
        try {
            CropFieldManager cropFieldManager = Lookup.getDefault().lookup(CropFieldManager.class);
            FileObject unoccupiedFolder = getUnoccupiedFolder(fieldPlanFolder);
            for (CropField cropField : cropFields) {
                cropFieldManager.save(cropField, unoccupiedFolder);
            }
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
            LOGGER.log(Level.SEVERE, "Failed to add fieldplan: {0}", ex.getMessage());
        }
        return fieldPlanFolder;
    }

    private FileObject getUnoccupiedFolder(FileObject fieldPlanFolder) throws IOException {
        FileObject unoccupiedFolder = fieldPlanFolder.getFileObject(UNOCCUPIED_FOLDER_NAME);
        if (unoccupiedFolder == null) {
            unoccupiedFolder = fieldPlanFolder.createFolder(UNOCCUPIED_FOLDER_NAME);
            CropProductionUnit cpu = new CropProductionUnit();
            cpu.setName(UNOCCUPIED_FOLDER_NAME);
            save(cpu, unoccupiedFolder);
        }
        return unoccupiedFolder;
    }


    @Override
    public FileObject getFieldPlansFolder() {
        return fieldPlansFolder.getPrimaryFile();
    }

    @Override
    public CropProductionUnit loadCropProductionUnit(FileObject cropProductionUnitFolder) throws IOException {
        LOGGER.log(Level.FINEST, "loading crop production unit data from folder {0}", cropProductionUnitFolder.getName());
        CropProductionUnit cropProductionUnit = null;
        FileObject dataObject = cropProductionUnitFolder.getFileObject(PRODUCTION_UNIT_OBJECT_NAME);
        assert dataObject != null : "specified folder did not contain a valid data object";
        try (ObjectInputStream ois = new ObjectInputStream(cropProductionUnitFolder.getFileObject(PRODUCTION_UNIT_OBJECT_NAME).getInputStream())) {
            cropProductionUnit = (CropProductionUnit) ois.readObject();
        } catch (ClassNotFoundException | IOException ex) {
            Exceptions.printStackTrace(ex);
            throw new DataObjectNotFoundException(dataObject);
        }
        return cropProductionUnit;
    }

    @Override
    public void save(CropProductionUnit cropProductionUnit, FileObject productionUnitFolder) throws IOException {
        LOGGER.log(Level.FINEST, "saving crop production unit data to folder {0}", productionUnitFolder.getName());
        try (ObjectOutputStream oos = new ObjectOutputStream(productionUnitFolder.createData(PRODUCTION_UNIT_OBJECT_NAME).getOutputStream())) {
            oos.writeObject(cropProductionUnit);
        }
    }

    @Override
    public FileObject addCropProductionUnit(String name, CropDefinition crop, FileObject fieldPlanFolder) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
