/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm.fieldplan.wizard;

import java.awt.Component;
import java.awt.Dialog;
import java.io.File;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.farm.model.FieldPlanManager;
import nl.cloudfarming.client.farm.fieldplan.FieldPlanManagerImpl;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.field.model.Field;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.spi.project.ui.support.ProjectChooser;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataFolder;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * Wizard iterator for creating a new fieldplan.
 *
 * The wizard will start at the fieldplans folder as initiated by the new
 * fieldplan action. This folder will also be the destination in finished
 *
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NbBundle.Messages({
    "LBL_DetailsProjectStep=Administration",
    "LBL_FieldsProjectStep=Add fields",
    "LBL_summaryProjectStep=Overview",
    "field_plan.wizard.title=New field plan"
})
public class FieldPlanWizardIterator implements WizardDescriptor.Iterator {

    private int index;
    private WizardDescriptor.Panel[] panels;
    private WizardDescriptor wizardDescriptor;
    private static final String TITLE = NbBundle.getMessage(FieldPlanWizardIterator.class, "field_plan.wizard.title");
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.farm.fieldplan.wizard.FieldPlanWizardIterator");
    public static final ImageIcon FIELDPLAN_ICON = new ImageIcon(ImageUtilities.loadImage("nl/cloudfarming/client/farm/fieldplan/plant-icon32.png"));

    public FieldPlanWizardIterator() {
        wizardDescriptor = new WizardDescriptor(this);
    }

    /**
     * Called from actionperformed to create a fieldplan in the context of a
     * project. OR called when checkbox in new Farm wizard has been checked to
     * create fieldplan after wizard finishes.
     *
     * @param destination (Project-)Folder to create fieldplan in
     */
    public void startWizard(FileObject destination) {
        // {0} will be replaced by WizardDescriptor.Panel.getComponent().getName()
        wizardDescriptor.setTitleFormat(new MessageFormat("{0}"));
        wizardDescriptor.setTitle(TITLE);
        getPanels();
        Dialog dialog = DialogDisplayer.getDefault().createDialog(wizardDescriptor);
        dialog.setVisible(true);
        dialog.toFront();
        finish(destination);
    }

    /**
     * Called from startWizard after wizard finishes
     *
     * @param destination (Project-)Folder to create field plan in
     */
    private void finish(FileObject destination) {

        boolean cancelled = wizardDescriptor.getValue() != WizardDescriptor.FINISH_OPTION;
        if (!cancelled) {
            File destinationFile = FileUtil.toFile(destination);
            if (destinationFile != null && destinationFile.exists()) {
                String name = wizardDescriptor.getProperty(FieldPlanDetailsWizardPanelVisual.PROP_FIELD_PLAN_NAME).toString();
                int year = Integer.parseInt(wizardDescriptor.getProperty(FieldPlanDetailsWizardPanelVisual.PROP_FIELD_PLAN_YEAR).toString());
                List<Field> selectedFieldsForFieldPlan = (List<Field>) wizardDescriptor.getProperty(FieldPlanFieldsWizardPanelVisual.PROP_SELECTED_FIELDS);
                List<CropField> cropFields = new ArrayList<>();
                for (Field field : selectedFieldsForFieldPlan) {
                    CropField cropField = new CropField(field);
                    cropFields.add(cropField);
                }

                DataFolder fieldPlansFolder = DataFolder.findFolder(destination);
                if (fieldPlansFolder.isValid()) {
                    try {
                        FieldPlanManager fieldPlanManager = new FieldPlanManagerImpl(fieldPlansFolder);
                        fieldPlanManager.addFieldPlan(name, year, cropFields);
                    } catch (IOException | IllegalArgumentException ex) {
                        LOGGER.log(Level.SEVERE, "Failed to add a new field plan {0}", ex.getMessage());
                        Exceptions.printStackTrace(ex);
                    }
                } else {
                    LOGGER.log(Level.SEVERE, "destination {0} is not a valid data folder", fieldPlansFolder.getName());
                }
            }
        }
    }

    private String[] createSteps() {
        return new String[]{NbBundle.getMessage(FieldPlanWizardIterator.class, "LBL_DetailsProjectStep"),
                    NbBundle.getMessage(FieldPlanWizardIterator.class, "LBL_FieldsProjectStep"),
                    NbBundle.getMessage(FieldPlanWizardIterator.class, "LBL_summaryProjectStep")};
    }

    /**
     * Initialize panels representing individual wizard's steps and sets various
     * properties for them influencing wizard appearance.
     */
    public WizardDescriptor.Panel[] getPanels() {
        if (panels == null) {
            panels = new WizardDescriptor.Panel[]{
                new FieldPlanDetailsWizardPanel(),
                new FieldPlanFieldsWizardPanel(),
                new FieldPlanSummaryPanel()
            };
            String[] steps = createSteps();
            for (int i = 0; i < panels.length; i++) {
                Component c = panels[i].getComponent();
                // Default step name to component name of panel. Mainly useful
                // for getting the name of the target chooser to appear in the
                // list of steps.
                steps[i] = c.getName();
                if (c instanceof JComponent) { // assume Swing components
                    JComponent jc = (JComponent) c;
                    // Sets step number of a component

                    jc.putClientProperty("WizardPanel_contentSelectedIndex", Integer.valueOf(i));
                    // Sets steps names for a panel
                    jc.putClientProperty("WizardPanel_contentData", steps);
                    // Turn on subtitle creation on each step
                    jc.putClientProperty("WizardPanel_autoWizardStyle", Boolean.TRUE);
                    // Show steps on the left side with the image on the background
                    jc.putClientProperty("WizardPanel_contentDisplayed", Boolean.TRUE);
                    // Turn on numbering of all steps
                    jc.putClientProperty("WizardPanel_contentNumbered", Boolean.TRUE);
                }
            }
        }
        return panels;
    }

    @Override
    public String name() {
        return MessageFormat.format("{0} of {1}",
                new Object[]{new Integer(index + 1), new Integer(panels.length)});
    }

    @Override
    public boolean hasNext() {
        return index < panels.length - 1;
    }

    @Override
    public boolean hasPrevious() {
        return index > 0;
    }

    @Override
    public void nextPanel() {
        if (!hasNext()) {
            throw new NoSuchElementException();
        }
        index++;
    }

    @Override
    public void previousPanel() {
        if (!hasPrevious()) {
            throw new NoSuchElementException();
        }
        index--;
    }

    @Override
    public WizardDescriptor.Panel current() {
        return panels[index];
    }

    // If nothing unusual changes in the middle of the wizard, simply:
    @Override
    public final void addChangeListener(ChangeListener l) {
    }

    @Override
    public final void removeChangeListener(ChangeListener l) {
    }
}
